@@ -1,7 +1,7 @@
  /*
  * gnote
  *
- * Copyright (C) 2010-2017,2019-2020 Aurimas Cernius
+ * Copyright (C) 2010-2017,2019-2021 Aurimas Cernius
  * Copyright (C) 2009 Hubert Figuiere
  *
  * This program is free software: you can redistribute it and/or modify
@@ -68,13 +68,13 @@ namespace gnote {
 
       Gtk::Button *button;
 
-      button = manage(new Gtk::Button(Gtk::Stock::CANCEL));
+      button = manage(new Gtk::Button(_("_Cancel"), true));
       button->property_can_default().set_value(true);
       button->show ();
       dialog.add_action_widget(*button, Gtk::RESPONSE_CANCEL);
       dialog.set_default_response(Gtk::RESPONSE_CANCEL);
 
-      button = manage(new Gtk::Button (Gtk::Stock::DELETE));
+      button = manage(new Gtk::Button(_("_Delete"), true));
       button->property_can_default().set_value(true);
       button->show ();
       dialog.add_action_widget(*button, 666);
@@ -396,6 +396,13 @@ namespace gnote {
   void Note::on_buffer_mark_set(const Gtk::TextBuffer::iterator & iter,
                                 const Glib::RefPtr<Gtk::TextBuffer::Mark> & insert)
   {
+    auto buffer = get_buffer();
+    auto insert_mark = buffer->get_insert();
+    auto selection_mark = buffer->get_selection_bound();
+    if(!(insert == insert_mark || insert == selection_mark)) {
+      return;
+    }
+
     Gtk::TextIter start, end;
     if(m_buffer->get_selection_bounds(start, end)) {
       m_data.data().set_cursor_position(start.get_offset());
@@ -404,16 +411,17 @@ namespace gnote {
     else if(insert->get_name() == "insert") {
       m_data.data().set_cursor_position(iter.get_offset());
     }
-    else {
-      return;
-    }
 
     DBG_OUT("OnBufferSetMark queueing save");
     queue_save(NO_CHANGE);
   }
 
-  void Note::on_buffer_mark_deleted(const Glib::RefPtr<Gtk::TextBuffer::Mark> &)
+  void Note::on_buffer_mark_deleted(const Glib::RefPtr<Gtk::TextBuffer::Mark> & mark)
   {
+    if(mark != get_buffer()->get_selection_bound()) {
+      return;
+    }
+
     Gtk::TextIter start, end;
     if(m_data.data().selection_bound_position() != m_data.data().cursor_position()
        && !m_buffer->get_selection_bounds(start, end)) {
