@@ -56,19 +56,21 @@ namespace gnote {
     , m_search_box(nullptr)
     , m_find_next_prev_box(nullptr)
     , m_search_entry(nullptr)
+    , m_embedded_widget(nullptr)
     , m_mapped(false)
     , m_entry_changed_timeout(NULL)
     , m_window_menu_embedded(NULL)
     , m_window_menu_default(NULL)
-    , m_keybinder(get_accel_group())
+    , m_accel_group(Gtk::AccelGroup::create())
+    , m_keybinder(m_accel_group)
   {
+    add_accel_group(m_accel_group);
     set_default_size(450,400);
     set_resizable(true);
     if(g.preferences().main_window_maximized()) {
       maximize();
     }
 
-    set_has_resize_grip(true);
     set_icon_name(IconManager::GNOTE);
 
     m_search_notes_widget = new SearchNotesWidget(g, m);
@@ -157,7 +159,7 @@ namespace gnote {
     m_all_notes_button->set_image(*image);
     m_all_notes_button->set_tooltip_text(_("All Notes"));
     m_all_notes_button->signal_clicked().connect(sigc::mem_fun(*this, &NoteRecentChanges::on_all_notes_button_clicked));
-    m_all_notes_button->add_accelerator("activate", get_accel_group(), GDK_KEY_comma, Gdk::CONTROL_MASK, (Gtk::AccelFlags) 0);
+    m_all_notes_button->add_accelerator("activate", m_accel_group, GDK_KEY_comma, Gdk::CONTROL_MASK, (Gtk::AccelFlags) 0);
     m_all_notes_button->show_all();
     left_box->attach(*m_all_notes_button, 0, 0, 1, 1);
 
@@ -167,14 +169,15 @@ namespace gnote {
     image->property_icon_size() = GTK_ICON_SIZE_MENU;
     m_new_note_button->set_image(*image);
     m_new_note_button->set_tooltip_text(_("Create New Note"));
-    m_new_note_button->add_accelerator("activate", get_accel_group(), GDK_KEY_N, Gdk::CONTROL_MASK, (Gtk::AccelFlags) 0);
+    m_new_note_button->add_accelerator("activate", m_accel_group, GDK_KEY_N, Gdk::CONTROL_MASK, (Gtk::AccelFlags) 0);
     m_new_note_button->signal_clicked().connect(sigc::mem_fun(*m_search_notes_widget, &SearchNotesWidget::new_note));
     m_new_note_button->show_all();
     left_box->attach(*m_new_note_button, 1, 0, 1, 1);
     left_box->show();
 
-    m_embedded_toolbar.set_margin_left(6);
-    m_embedded_toolbar.set(Gtk::ALIGN_START, Gtk::ALIGN_CENTER, 0, 0);
+    m_embedded_toolbar.set_margin_start(6);
+    m_embedded_toolbar.set_halign(Gtk::ALIGN_START);
+    m_embedded_toolbar.set_valign(Gtk::ALIGN_CENTER);
     m_embedded_toolbar.show();
 
     Gtk::Grid *right_box = manage(new Gtk::Grid);
@@ -186,7 +189,7 @@ namespace gnote {
     image->property_icon_size() = GTK_ICON_SIZE_MENU;
     m_search_button.set_image(*image);
     m_search_button.signal_toggled().connect(sigc::mem_fun(*this, &NoteRecentChanges::on_search_button_toggled));
-    m_search_button.add_accelerator("activate", get_accel_group(), GDK_KEY_F, Gdk::CONTROL_MASK, (Gtk::AccelFlags) 0);
+    m_search_button.add_accelerator("activate", m_accel_group, GDK_KEY_F, Gdk::CONTROL_MASK, (Gtk::AccelFlags) 0);
     m_search_button.set_tooltip_text(_("Search"));
     m_search_button.show_all();
     right_box->attach(m_search_button, 0, 0, 1, 1);
@@ -199,7 +202,7 @@ namespace gnote {
     m_window_actions_button->signal_clicked().connect(
       sigc::mem_fun(*this, &NoteRecentChanges::on_show_window_menu));
     m_window_actions_button->add_accelerator(
-      "activate", get_accel_group(), GDK_KEY_F10, (Gdk::ModifierType) 0, (Gtk::AccelFlags) 0);
+      "activate", m_accel_group, GDK_KEY_F10, (Gdk::ModifierType) 0, (Gtk::AccelFlags) 0);
     m_window_actions_button->show_all();
     right_box->attach(*m_window_actions_button, 1, 0, 1, 1);
     right_box->show();
@@ -214,8 +217,8 @@ namespace gnote {
     }
     else {
       Gtk::Grid *header_bar = manage(new Gtk::Grid);
-      header_bar->set_margin_left(5);
-      header_bar->set_margin_right(5);
+      header_bar->set_margin_start(5);
+      header_bar->set_margin_end(5);
       header_bar->set_margin_top(5);
       header_bar->set_margin_bottom(5);
       header_bar->attach(*left_box, 0, 0, 1, 1);
@@ -251,16 +254,10 @@ namespace gnote {
       .connect(sigc::mem_fun(*this, &NoteRecentChanges::on_entry_activated));
     m_search_entry->show();
 
-    Gtk::Grid *grid = manage(new Gtk::Grid);
-    grid->set_margin_left(5);
-    grid->set_margin_right(5);
-    grid->set_hexpand(false);
-    grid->attach(*m_search_entry, 0, 0, 1, 1);
-    grid->show();
-
-    m_search_box = manage(new Gtk::Alignment(0.5, 0.5, 0.0, 1.0));
-    m_search_box->add(*grid);
-    m_search_box->set_hexpand(true);
+    m_search_box = manage(new Gtk::Grid);
+    m_search_box->set_hexpand(false);
+    m_search_box->attach(*m_search_entry, 0, 0, 1, 1);
+    m_search_box->set_halign(Gtk::ALIGN_CENTER);
 
     auto content = dynamic_cast<Gtk::Grid*>(m_embed_box.get_parent());
     if(content) {
@@ -278,7 +275,7 @@ namespace gnote {
     }
 
     m_find_next_prev_box = manage(new Gtk::Grid);
-    m_find_next_prev_box->set_margin_left(5);
+    m_find_next_prev_box->set_margin_start(5);
 
     Gtk::Button *find_next_button = manage(new Gtk::Button);
     Gtk::Image *image = manage(new Gtk::Image);
@@ -288,7 +285,7 @@ namespace gnote {
     find_next_button->set_always_show_image(true);
     find_next_button->signal_clicked()
       .connect(sigc::mem_fun(*this, &NoteRecentChanges::on_find_next_button_clicked));
-    find_next_button->add_accelerator("activate", get_accel_group(), GDK_KEY_G, Gdk::CONTROL_MASK, (Gtk::AccelFlags) 0);
+    find_next_button->add_accelerator("activate", m_accel_group, GDK_KEY_G, Gdk::CONTROL_MASK, (Gtk::AccelFlags) 0);
     find_next_button->show();
     m_find_next_prev_box->attach(*find_next_button, 0, 0, 1, 1);
 
@@ -300,7 +297,7 @@ namespace gnote {
     find_prev_button->set_always_show_image(true);
     find_prev_button->signal_clicked()
       .connect(sigc::mem_fun(*this, &NoteRecentChanges::on_find_prev_button_clicked));
-    find_prev_button->add_accelerator("activate", get_accel_group(), GDK_KEY_G, Gdk::CONTROL_MASK|Gdk::SHIFT_MASK, (Gtk::AccelFlags) 0);
+    find_prev_button->add_accelerator("activate", m_accel_group, GDK_KEY_G, Gdk::CONTROL_MASK|Gdk::SHIFT_MASK, (Gtk::AccelFlags) 0);
     find_prev_button->show();
     m_find_next_prev_box->attach(*find_prev_button, 1, 0, 1, 1);
 
@@ -320,7 +317,7 @@ namespace gnote {
     }
     else {
       m_search_box->hide();
-      SearchableItem *searchable_widget = dynamic_cast<SearchableItem*>(currently_embedded());
+      SearchableItem *searchable_widget = dynamic_cast<SearchableItem*>(currently_foreground());
       if(searchable_widget) {
         searchable_widget->perform_search("");
       }
@@ -329,7 +326,7 @@ namespace gnote {
 
   void NoteRecentChanges::on_find_next_button_clicked()
   {
-    SearchableItem *searchable_widget = dynamic_cast<SearchableItem*>(currently_embedded());
+    SearchableItem *searchable_widget = dynamic_cast<SearchableItem*>(currently_foreground());
     if(searchable_widget) {
       searchable_widget->goto_next_result();
     }
@@ -337,7 +334,7 @@ namespace gnote {
 
   void NoteRecentChanges::on_find_prev_button_clicked()
   {
-    SearchableItem *searchable_widget = dynamic_cast<SearchableItem*>(currently_embedded());
+    SearchableItem *searchable_widget = dynamic_cast<SearchableItem*>(currently_foreground());
     if(searchable_widget) {
       searchable_widget->goto_previous_result();
     }
@@ -354,7 +351,7 @@ namespace gnote {
       m_search_entry->grab_focus();
     }
     Glib::ustring text = m_search_entry->get_text();
-    update_search_bar(*currently_embedded(), text != "");
+    update_search_bar(*currently_foreground(), text != "");
   }
 
   void NoteRecentChanges::update_search_bar(EmbeddableWidget & widget, bool perform_search)
@@ -385,7 +382,7 @@ namespace gnote {
 
   void NoteRecentChanges::present_search()
   {
-    EmbeddableWidget *current = currently_embedded();
+    EmbeddableWidget *current = currently_foreground();
     if(m_search_notes_widget == dynamic_cast<SearchNotesWidget*>(current)) {
       return;
     }
@@ -453,6 +450,10 @@ namespace gnote {
         background_embedded(*widget);
       }
     }
+    if(m_embedded_widget) {
+      m_embedded_widget->unembed();
+      m_embedded_widget = nullptr;
+    }
 
     hide();
   }
@@ -460,7 +461,7 @@ namespace gnote {
 
   bool NoteRecentChanges::is_search()
   {
-    return m_search_notes_widget == currently_embedded();
+    return m_search_notes_widget == currently_foreground();
   }
 
 
@@ -491,7 +492,7 @@ namespace gnote {
         close_window();
       }
       else if(m_preferences.enable_close_note_on_escape()) {
-        EmbeddableWidget *current_item = currently_embedded();
+        EmbeddableWidget *current_item = currently_foreground();
         if(current_item) {
           background_embedded(*current_item);
         }
@@ -513,11 +514,8 @@ namespace gnote {
     // Select "All Notes" in the notebooks list
     m_search_notes_widget->select_all_notes_notebook();
 
-    EmbeddableWidget *widget = NULL;
-    if(m_embed_box.get_children().size() == 0 && m_embedded_widgets.size() > 0) {
-      widget = *m_embedded_widgets.rbegin();
-      foreground_embedded(*widget);
-    }
+    EmbeddableWidget *widget = m_embedded_widget ? m_embedded_widget : m_search_notes_widget;
+    foreground_embedded(*widget);
 
     MainWindow::on_show();
 
@@ -547,18 +545,15 @@ namespace gnote {
 
   void NoteRecentChanges::embed_widget(EmbeddableWidget & widget)
   {
-    if(std::find(m_embedded_widgets.begin(), m_embedded_widgets.end(), &widget) == m_embedded_widgets.end()) {
-      widget.embed(this);
-      m_embedded_widgets.push_back(&widget);
-    }
-    EmbeddableWidget *current = currently_embedded();
+    EmbeddableWidget *current = currently_foreground();
     if(current == &widget) {
       return;
     }
-    if(current) {
-      background_embedded(*current);
+    if(m_embedded_widget) {
+      unembed_widget(*m_embedded_widget);
     }
-
+    m_embedded_widget = &widget;
+    widget.embed(this);
    if(get_visible()) {
       foreground_embedded(widget);
     }
@@ -566,32 +561,30 @@ namespace gnote {
 
   void NoteRecentChanges::unembed_widget(EmbeddableWidget & widget)
   {
-    bool show_other = false;
-    auto iter = std::find(m_embedded_widgets.begin(), m_embedded_widgets.end(), &widget);
-    if(iter != m_embedded_widgets.end()) {
-      if(is_foreground(**iter)) {
+    bool show_search = false;
+    if(&widget == m_embedded_widget) {
+      if(is_foreground(widget)) {
         background_embedded(widget);
-        show_other = true;
+        show_search = true;
       }
-      m_embedded_widgets.erase(iter);
+      m_embedded_widget = nullptr;
       widget.unembed();
     }
-    if(show_other) {
-      if(m_embedded_widgets.size()) {
-	foreground_embedded(**m_embedded_widgets.rbegin());
-      }
-      else if(get_visible()) {
-        close_window();
-      }
+    if(show_search) {
+      foreground_embedded(*m_search_notes_widget);
     }
   }
 
   void NoteRecentChanges::foreground_embedded(EmbeddableWidget & widget)
   {
     try {
-      if(currently_embedded() == &widget) {
+      EmbeddableWidget *current_foreground = currently_foreground();
+      if(current_foreground == &widget) {
         return;
       }
+      else if(current_foreground) {
+        background_embedded(*current_foreground);
+      }
       Gtk::Widget &wid = dynamic_cast<Gtk::Widget&>(widget);
       m_embed_box.add(wid);
       wid.show();
@@ -640,7 +633,7 @@ namespace gnote {
   void NoteRecentChanges::background_embedded(EmbeddableWidget & widget)
   {
     try {
-      if(currently_embedded() != &widget) {
+      if(currently_foreground() != &widget) {
         return;
       }
       Gtk::Widget &wid = dynamic_cast<Gtk::Widget&>(widget);
@@ -656,18 +649,19 @@ namespace gnote {
     catch(std::bad_cast&) {
     }
 
-    m_embedded_toolbar.remove();
+    auto children = m_embedded_toolbar.get_children();
+    for(auto child : children) {
+      m_embedded_toolbar.remove(*child);
+    }
   }
 
   bool NoteRecentChanges::contains(EmbeddableWidget & widget)
   {
-    for(EmbeddableWidget *wgt : m_embedded_widgets) {
-      if(dynamic_cast<EmbeddableWidget*>(wgt) == &widget) {
-        return true;
-      }
+    if(&widget == m_search_notes_widget) {
+      return true;
     }
 
-    return false;
+    return &widget == m_embedded_widget;
   }
 
   bool NoteRecentChanges::is_foreground(EmbeddableWidget & widget)
@@ -705,7 +699,7 @@ namespace gnote {
     }
   }
 
-  EmbeddableWidget *NoteRecentChanges::currently_embedded()
+  EmbeddableWidget *NoteRecentChanges::currently_foreground()
   {
     std::vector<Gtk::Widget*> children = m_embed_box.get_children();
     return children.size() ? dynamic_cast<EmbeddableWidget*>(children[0]) : NULL;
@@ -715,7 +709,7 @@ namespace gnote {
   {
     bool res = MainWindow::on_map_event(evt);
     if(!m_mapped) {
-      auto widget = currently_embedded();
+      auto widget = currently_foreground();
       if(widget) {
         widget->set_initial_focus();
       }
@@ -750,7 +744,7 @@ namespace gnote {
 
     Glib::ustring search_text = get_search_text();
     if(search_text.empty()) {
-      SearchableItem *searchable_widget = dynamic_cast<SearchableItem*>(currently_embedded());
+      SearchableItem *searchable_widget = dynamic_cast<SearchableItem*>(currently_foreground());
       if(searchable_widget) {
         searchable_widget->perform_search(search_text);
       }
@@ -779,7 +773,7 @@ namespace gnote {
       return;
     }
 
-    SearchableItem *searchable_widget = dynamic_cast<SearchableItem*>(currently_embedded());
+    SearchableItem *searchable_widget = dynamic_cast<SearchableItem*>(currently_foreground());
     if(searchable_widget) {
       searchable_widget->perform_search(search_text);
     }
@@ -825,7 +819,7 @@ namespace gnote {
 
   void NoteRecentChanges::on_show_window_menu()
   {
-    HasActions *embed_with_actions = dynamic_cast<HasActions*>(currently_embedded());
+    HasActions *embed_with_actions = dynamic_cast<HasActions*>(currently_foreground());
     if(embed_with_actions) {
       if(m_window_menu_embedded == NULL) {
         m_window_menu_embedded = make_window_menu(m_window_actions_button, std::move(embed_with_actions->get_popover_widgets()));
